<?php
/************************************************************
* Plugin Name:	Events Manager – OpenStreetMaps
* Plugin URI: 	https://wordpress.org/plugins/stonehenge-em-osm/
* Description:	OpenStreetMaps replacement for Events Manager.
* Version:		2.6.3
* Author:  		Stonehenge Creations
* Author URI: 	https://www.stonehengecreations.nl/
* License URI: 	https://www.gnu.org/licenses/gpl-2.0.html
* Text Domain: 	stonehenge-em-osm
* Domain Path: 	/languages
************************************************************/

// Exit if accessed directly
if (!defined('ABSPATH')) exit;

include_once(ABSPATH.'wp-admin/includes/plugin.php');


#===============================================
# Check for Events Manager.
#===============================================
if( !is_plugin_active('events-manager/events-manager.php') ) {
	deactivate_plugins(plugin_basename(__FILE__));
	wp_die('<b>EM OpenStreetMaps</b> needs Events Manager to be installed and activated.<br>This plugin has been deactivated...', 'Plugin dependency check', array('back_link' => true));
}
else {
	require_once('config/init.php');
	new Stonehenge_EM_OSM();
}


Class Stonehenge_EM_OSM {

	private $version = '2.6.3';

	public function __construct() {
		// Common Plugin Actions.
		register_deactivation_hook(__FILE__, array( $this, 'remove_template_files'));
		add_action('init', array($this, 'load_translations'));

		// Admin Functions.
		add_action('admin_notices', array($this, 'create_admin_notices') );
		add_action('wp_loaded', array($this, 'rename_template_event'), 11, 1);
		add_action('wp_loaded', array($this, 'rename_template_location'), 12, 1);

		// Core alterations.
		add_action('wp_loaded', array($this, 'add_em_ajax_results'), 33);
		add_action('wp_loaded', array($this, 'disable_google_api'), 50);

		add_action('wp_enqueue_scripts', array($this, 'replace_em_js'));
		add_action('admin_enqueue_scripts', array($this, 'replace_em_js'));
		add_action('wp_loaded', array($this, 'restore_original_em_js'), 13, 1);

		include('em-osm-maps.php');
		new Stonehenge_EM_OSM_Maps;
		new EM_Addon( self::get_plugin_data() );
	}


	#===============================================
	# 	This defines the Admin side of this Plugin.
	#===============================================
	public static function get_plugin_data() {
		$plugin = array(
			'version' 	=> '2.6.3',
			'class' 	=> 'Stonehenge_EM_OSM',
			'name' 		=> 'Events Manager - OpenStreetMaps',
			'short' 	=> 'OpenStreetMaps',
			'slug' 		=> 'stonehenge_em_osm',
			'text' 		=> 'stonehenge-em-osm',
			'icon' 		=> '&#x1F5FA;',
			'prio' 		=> 40,
		);
		$plugin['url'] 	= admin_url().'admin.php?page='.$plugin['slug'];
		return $plugin;
	}


	#===============================================
	# 	Add (old) values to the Defaults.
	#===============================================
	public static function add_defaults() {
		$old_values = get_option('stonehenge-em-osm');
		if( $old_values && !empty($old_values) && is_array($old_values) ) {
			delete_option('stonehenge-em-osm');
			return $old_values;
		}
		return;
	}


	#===============================================
	# 	Perform update if needed.
	# 	This runs alone, so anything can be done.
	#===============================================
	public static function update_this_plugin() {
		// Do stuff.
		$plugin = self::get_plugin_data();

		EM_Addon::show_new_options_notice($plugin, 'info');
		return;
	}


	#===============================================
	# 	Sanitize form inputs before saving to the database.
	#===============================================
	public static function sanitize_option_fields( $input ) {
		$sections 	= self::define_option_fields();
		$clean 		= EM_Addon::sanitize_option_fields( $input, $sections );
		return $clean;
	}


	#===============================================
	# 	Load Styles & Scripts.
	#===============================================
	public static function load_addon_assets()  {
		$addon = self::get_plugin_data();
		EM_Addon::load_main_assets();
	}


	#===============================================
	# 	Make this Addon translatable.
	#===============================================
	public static function load_translations() {
		$addon 	= self::get_plugin_data();
		$text 	= $addon['text'];

		// Try included files first.
		$locale = apply_filters( 'plugin_locale', function_exists( 'determine_locale' ) ? determine_locale() : get_locale(), $text );
		$mofile = dirname( __FILE__ ) . '/languages/'. $text . '-' . $locale . '.mo';
		$loaded = load_textdomain( $text, $mofile );

		// Else, look for the WP Translations.
		if( !$loaded ) {
			$loaded = load_plugin_textdomain( $text, false, '/languages/' );
		}

		// MU Plugin perhaps?
		if( !$loaded ) {
			$loaded = load_muplugin_textdomain( $text, '/languages/' );
		}
	}


	#===============================================
	# 	Define this Plugins Options.
	#===============================================
	public static function define_option_fields() {
		$plugin		= self::get_plugin_data();
		$text  		= $plugin['text'];
		$sections[] = array(
			'id' 		=> 'osm',
			'label'		=> __('Settings'),
			'fields' 	=> array(
				array(
					'id' 		=> 'intro',
					'label' 	=> 'Intro',
					'type'		=> 'intro',
					'default' 	=> __('This plugin completely replaces the built-in Google Maps with open source OpenStreetMap.', $text) .'<br>'.
			__('It uses OpenCage for geocoding.', $text),
				),
				array(
					'id' 		=> 'api',
					'label' 	=> __('OpenCage API Key', $text),
					'type' 		=> 'text',
					'required' 	=> true,
					'size'		=> 'medium',
					'after' 	=> sprintf( '<a href=%s target="_blank">%s</a>', 'https://opencagedata.com/pricing', '<button type="button" class="stonehenge-button">'. __('Get your free key', $text) .'</button>'),
					'helper' 	=> __('A free OpenCage API Key has a daily limit of 2500 calls per day.', $text) .'<br>'. __('This plugin only calls the OpenCage API when you click the "Search Address" button.', $text),
				),

				array(
					'id' 		=> 'zoom',
					'label' 	=> __('Single Marker Zoom', $text),
					'type' 		=> 'number',
					'required'	=> true,
					'min' 		=> '0',
					'max' 		=> '19',
					'default'	=> '15',
					'helper' 	=> __('Enter a number between 1 and 19. (Default is 15)', $text),
				),

				array(
					'id' 		=> 'marker',
					'label'		=> __('Marker Color', $text),
					'required' 	=> true,
					'type' 		=> 'select',
					'choices' 	=> array(
						'blue' 		=> __('Blue', $text),
						'red' 		=> __('Red', $text),
						'green' 	=> __('Green', $text),
						'orange' 	=> __('Orange', $text),
						'yellow' 	=> __('Yellow', $text),
						'violet' 	=> __('Purple', $text),
						'grey' 		=> __('Grey', $text),
						'black' 	=> __('Black', $text),
					),
				),
				array(
					'id' 		=> 'type',
					'label' 	=> __('Map Style', $text),
					'type' 		=> 'select',
					'choices' 	=> array(
						'//{s}.tile.openstreetmap.org/{z}/{x}/{y}.png' 	=> 'OpenStreetMap',
						'//{s}.tile.openstreetmap.fr/hot/{z}/{x}/{y}.png' => 'OpenStreetMap HOT',
						'//server.arcgisonline.com/ArcGIS/rest/services/World_Street_Map/MapServer/tile/{z}/{y}/{x}' => 'ArcGIS WorldMap',
						'//server.arcgisonline.com/ArcGIS/rest/services/World_Topo_Map/MapServer/tile/{z}/{y}/{x}' => 'ArcGIS TopoMap',
						'//server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile/{z}/{y}/{x}' => 'ArcGIS World Imagery',
						'//{s}.tile.openstreetmap.se/hydda/full/{z}/{x}/{y}.png' => 'Hydda (max zoom = 18)',
					),
					'helper' 	=> __('You can preview map styles by reloading any map on your website.', $text),
					'required' 	=> true,
				),
				array(
					'id' 		=> 'zoomcontrols',
					'label' 	=> __('Show Zoom Controls', $text),
					'type' 		=> 'toggle',
					'default' 	=> 'no',
					'required' 	=> true,
				),
				array(
					'id' 		=> 'fullscreen',
					'label' 	=> __('Show Fullscreen', $text),
					'type' 		=> 'toggle',
					'default' 	=> 'no',
					'required' 	=> true,
				),
				array(
					'id' 		=> 'scale',
					'label' 	=> __('Show Scale', $text),
					'type' 		=> 'toggle',
					'default' 	=> 'no',
					'required' 	=> true,
				),
				array(
					'id' 		=> 'delete',
					'label' 	=> __('Delete Data', $text),
					'type' 		=> 'toggle',
					'required'	=> true,
					'helper' 	=> __('Automatically delete all data from your database when you uninstall this plugin?', $text),
					'default' 	=> 'yes',
				),
			),
		);
		return $sections;
	}


	#===============================================
	# 	Show the Location MetaBox in Edit Location Pages.
	#===============================================
	public static function show_osm_meta_box() {
		global $EM_Location, $EM_Event, $post;
		$required 	= apply_filters('em_required_html','<i>*</i>');
		?>
		<input type="hidden" name="_emnonce" value="<?php echo wp_create_nonce('edit_location'); ?>">
		<div id="em-location-data" class="em-location-data">
			<table id="osm-location-table" class="em-location-data">
				<?php include('partials/location-form-fields.php'); ?>
			<br style="clear:both;">
		</div>
		<?php
	}


	// Deprecated: Workaround for templates not being replaced.
	public static function construct_admin_map() {
		global $EM_Event;
		return self::show_edit_event_box( $EM_Event );
	}


	#===============================================
	# 	Show the Location MetaBox in Edit Event Pages.
	#===============================================
	public static function show_edit_event_box( $EM_Event ) {
		global $EM_Event;
		$required = apply_filters('em_required_html','<i>*</i>');

		//------------------------------------------------------
		// 	If "No Location" is checked.
		//------------------------------------------------------
		if( !get_option('dbem_require_location') && !get_option('dbem_use_select_for_locations') ) {
			?><div class="em-location-data-nolocation">
				<p>
					<label for="no-location"><input type="checkbox" name="no_location" id="no-location" value="1" <?php if( $EM_Event->location_id === '0' || $EM_Event->location_id === 0 ) echo 'checked="checked"'; ?>>&nbsp;<?php _e('This event does not have a physical location.','events-manager'); ?></label>
				</p>
			</div>
			<?php
		}

		//------------------------------------------------------
		// If a location is required, create a div for location details.
		//------------------------------------------------------
		?>
		<div id="em-location-data" class="em-location-data">
			<table id="osm-location-table" class="osm-location-table">
				<?php
				// 	If "Select Dropdown" is used.
				if( get_option('dbem_use_select_for_locations') || !$EM_Event->can_manage('edit_locations','edit_others_locations') ) {
					include('partials/location-select.php');
				}
				// 	If "Location Form Fields" are used.
				else {
					if( empty($EM_Event->location_id) ) {
						$EM_Location = (get_option('dbem_default_location') > 0) ? em_get_location(get_option('dbem_default_location')) : new EM_Location();
					}
					else {
						$EM_Location = $EM_Event->get_location();
					}
				?>
				<tr class="em-location-data-name">
					<th><?php _e ( 'Location Name:', 'events-manager')?>&nbsp;</th>
					<td><input id="location-name" type="text" name="location_name" value="<?php echo esc_attr($EM_Location->location_name, ENT_QUOTES); ?>" style="width:250px !important;"> <?php echo $required; ?>
					</td>
				</tr>
				<tr>
					<td colspan="2">
						<em id="em-location-search-tip"></em>
						<em id="em-location-reset" style="display:none;"><?php esc_html_e('You cannot edit saved locations here.', 'events-manager'); ?><br><a href="#"><?php esc_html_e('Reset this form to create a location or search again.', 'events-manager')?></a></em>
					</td>
				</tr>
				<?php
					include('partials/location-form-fields.php');
				}
			?>
			</table>
			<br style="clear:both;" />
		</div>
		<?php
	}


	#===============================================
	# 	Create a hint to use the search fields.
	#===============================================
	public static function show_search_tip() {
		$plugin 	= self::get_plugin_data();
		$text 		= $plugin['text'];
		$saved 		= get_option( $plugin['slug'] );
		$api 		= $saved['api'];

		if( !$api || empty($api) ) {
			$return 	= '<p style="color:#e14d43; font-weight: bold;">'. sprintf( __('Please enter your OpenCage API Key in your <a href=%s>Plugin Settings</a>.', $plugin['text']), $plugin['url'] ) .'</p>';

		}
		else {
			$button 	= '<button type="button" class="button-primary" onClick="apiSearch()">'. esc_html__('Search Address', $text) .'</button>';
			$expl 		= esc_html__('Hint', $text);
			$tooltip 	= esc_html__('If your location cannot be found, search for one nearby.', $text) .'<br>'. esc_html__('After the marker has been set, you can drag the marker to the preferred position and manually change the address details in the location form fields.', $text);

			$return 	= sprintf( '<div id="osm-search-tip" class="description"><span>%s %s</span></div>%s',
				esc_attr__('The more details you provide, the more accurate the search result will be.', $text),
				'<span class="osm-tooltip">('. $expl .')<span class="osm-tooltiptext">'. $tooltip .'</span></span>', $button
			);
		}
		return $return;
	}


	#===============================================
	# 	Replace events-manager.js with modified version.
	#===============================================
	function replace_em_js() {
		// Unload original JS.
		wp_dequeue_script('events-manager');
		wp_deregister_script('events-manager');

		// Define new version - without Google, with OSM.
	    $script_deps = array(
	    	'jquery'=>'jquery',
	    	'jquery-ui-core'=>'jquery-ui-core',
	    	'jquery-ui-datepicker'=>'jquery-ui-datepicker',
	    	'jquery-ui-autocomplete'=>'jquery-ui-autocomplete',
	    	'jquery-ui-dialog'=>'jquery-ui-dialog',
	    	'jquery-ui-widget'=>'jquery-ui-widget',
	    	'jquery-ui-position'=>'jquery-ui-position',
	    	'jquery-ui-sortable'=>'jquery-ui-sortable',
		);
		wp_register_script('events-manager', plugins_url('templates/osm-events-manager.min.js', __FILE__),  array_values($script_deps), EM_VERSION);
		wp_enqueue_script('events-manager');
		EM_Scripts_and_Styles::localize_script();
	}


	#===============================================
	# 	Restore original JS. Replaced with new enqueue_script.
	#===============================================
	public static function restore_original_em_js() {
		// Delete the modified JS file from previous versions and restore the original JS file.
		$file 			= 'events-manager.js';
		$EM_folder 		= WP_PLUGIN_DIR .'/events-manager/includes/js/';
		$EM_file 		= $EM_folder.$file;
		$old_file 		= $EM_folder.'_events-manager.js';

		if( file_exists($EM_file) && file_exists($old_file) ) {
			wp_delete_file($EM_file);
			rename($old_file, $EM_file);
			return;
		}

		if( !file_exists($EM_file) && file_exists($old_file) ) {
			rename($old_file, $EM_file);
			return;
		}
	}


	#===============================================
	# 	Add additional results to Ajax Locations Search.
	#===============================================
	public static function add_em_ajax_results() {
		$file 		= 'em-actions.php';
		$EM_folder 	= WP_PLUGIN_DIR .'/events-manager/';
		$EM_file 	= $EM_folder . $file;
		$OSM_folder = plugin_dir_path(__FILE__) . 'templates/';
		$OSM_file 	= $OSM_folder . $file;
		$old_file 	= $EM_folder . '_em-actions.php';

		// If only original file exists.
		if( file_exists($EM_file) && !file_exists($old_file) ) {
			rename($EM_file, $old_file);
			copy($OSM_file, $EM_file);
			return;
		}
		// If only altered file exists (= fallback).
		if( !file_exists($EM_file) && file_exists($old_file) ) {
			rename($old_file, $EM_file);
			return;
		}
		return;
	}


	#===============================================
	# 	Remove template files upon deactivation.
	#===============================================
	public static function remove_template_files() {
		$theme_location = get_stylesheet_directory().'/plugins/events-manager/forms/event/location.php';
		$theme_event 	= get_stylesheet_directory().'/plugins/events-manager/forms/location/where.php';
		if( file_exists($theme_location) ) {
			wp_delete_file($theme_location);
		}
		if( file_exists($theme_event) ) {
			wp_delete_file($theme_event);
		}
	}


	#===============================================
	# 	Show Admin Notice if OpenCage API is missing.
	#===============================================
	function create_admin_notices() {
		$plugin 	= self::get_plugin_data();
		$saved 		= get_option( $plugin['slug'] );
		if( !$saved['api'] ) {
			$message = '<div class="notice notice-error"><p>';
			$message .= sprintf( __('Please enter your OpenCage API Key in your <a href=%s>Plugin Settings</a>.', $plugin['text']), $plugin['url'] );
			$message .= '</p></div>';
			echo $message;
		}
		return;
	}


	#===============================================
	# 	Make sure the right templates are used.
	#===============================================
	public static function rename_template_event() {
		$version 		= '1.8.5';
		$file 			= 'where.php';
		$theme_folder 	= get_stylesheet_directory() . '/plugins/events-manager/forms/location/';
		$plugin_folder	= plugin_dir_path(__FILE__) . 'templates/';
		$plugin_file 	= $plugin_folder.$file;
		$theme_file 	= $theme_folder.$file;
		$version_file 	= $theme_folder.'where-'.$version.'.php';

		// Clean install.
		if( !is_dir($theme_folder) && !file_exists($theme_file) && !file_exists($version_file) ) {
			wp_mkdir_p($theme_folder);
			copy($plugin_file, $theme_file);
			rename($theme_file, $version_file);
			copy($plugin_file, $theme_file);
		}

		// If folder exists, but the files not.
		elseif( is_dir($theme_folder) && !file_exists($theme_file) && !file_exists($version_file) ) {
			copy($plugin_file, $theme_file);
			rename($theme_file, $version_file);
			copy($plugin_file, $theme_file);
		}

		// If folder and template exist, back-up old template.
		elseif( is_dir($theme_folder) && file_exists($theme_file) && !file_exists($version_file) ) {
			rename($theme_file, $version_file);
			copy($plugin_file, $theme_file);
		}

		// If folder and back-up exist, but main template is missing.
		elseif( is_dir($theme_folder) && !file_exists($theme_file) && file_exists($version_file) ) {
			copy($plugin_file, $theme_file);
		}

		else {
			return;
		}
	}

	public static function rename_template_location() {
		$version 		= '1.8.5';
		$file 			= 'location.php';
		$theme_folder 	= get_stylesheet_directory() . '/plugins/events-manager/forms/event/';
		$plugin_folder	= plugin_dir_path(__FILE__) . 'templates/';
		$plugin_file 	= $plugin_folder.$file;
		$theme_file 	= $theme_folder.$file;
		$version_file 	= $theme_folder.'location-'.$version.'.php';

		// Clean install.
		if( !is_dir($theme_folder) && !file_exists($theme_file) && !file_exists($version_file) ) {
			wp_mkdir_p($theme_folder);
			copy($plugin_file, $theme_file);
			rename($theme_file, $version_file);
			copy($plugin_file, $theme_file);
		}

		// If folder exists, but the files not.
		elseif( is_dir($theme_folder) && !file_exists($theme_file) && !file_exists($version_file) ) {
			copy($plugin_file, $theme_file);
			rename($theme_file, $version_file);
			copy($plugin_file, $theme_file);
		}

		// If folder and template exist, back-up old template.
		elseif( is_dir($theme_folder) && file_exists($theme_file) && !file_exists($version_file) ) {
			rename($theme_file, $version_file);
			copy($plugin_file, $theme_file);
		}

		// If folder and back-up exist, but main template is missing.
		elseif( is_dir($theme_folder) && !file_exists($theme_file) && file_exists($version_file) ) {
			copy($plugin_file, $theme_file);
		}

		else {
			return;
		}
	}


	#===============================================
	# 	Make sure Google Maps integration is deactivated.
	#===============================================
	function disable_google_api() {
		if( '0' != get_option('dbem_gmap_is_active') ) {
			update_option( 'dbem_gmap_is_active', '0' );
		}
	}

} // End Class.






/**/